from jnprsr import *
from jnprsr.cli.cliutils import _read_from_stdin
import argparse


def _argparser_simple_direct_input():
    parser = argparse.ArgumentParser(
        prog="jnprsr-pretty",
        description='Pretty prints a given Juniper Configuration read from STDIN. End input with CTRL+D or sequence \'!END\'',
        epilog="jnprsr is a Parser for Juniper Configuration Files"
    )
    parser.add_argument('-s', '--silent', action="store_true", help="Silent mode: Silences any additional output. Recommended when used in scripts")
    args = parser.parse_args()
    return args

def pretty():
    args = _argparser_simple_direct_input()

    input_data = _read_from_stdin(silent=args.silent)
    # We simply generate an abstract syntax tree
    ast = get_ast(input_data)
    # ... and rendering the config again based on that!
    out = render_config_from_ast(ast)
    print(out)





import unittest
import jnprsr.utils

class TestGetSubtree(unittest.TestCase):
    def read_test_file(self, name):
        with open(name, "r") as f:
            return f.read()

    def test_1(self):
        ast1 = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        ast2 = jnprsr.utils.get_sub_tree(ast1, "interfaces ge-0/0/0")
        out = jnprsr.utils.render_ascii_tree_from_ast(ast2)
        print(out)




import argparse

from jnprsr import *
import prompt_toolkit
from jnprsr.prompt_toolkit_custom_nested_completer import CustomNestedCompleter
from jnprsr.cli.cliutils import _read_from_file
from anytree.resolver import ChildResolverError



def _argparser_simple_one_file():
    parser = argparse.ArgumentParser(
        prog="jnprsr-subtree",
        description='Opens an interactive CLI allowing to inspect a given configuration file.',
        epilog="jnprsr is a Parser for Juniper Configuration Files"
    )
    parser.add_argument('filename', help="Configuration file to open")
    args = parser.parse_args()
    return args

def subtree():
    args = _argparser_simple_one_file()
    input_data = _read_from_file(args.filename)

    ast = get_ast(input_data)
    session = prompt_toolkit.PromptSession()

    # Transform AST into dict for completer
    completer_dict = {
        "show": {
            "configuration":
                render_dict_from_ast(ast)["root"]
        }
    }

    # Instantiate completer from dict
    completer = CustomNestedCompleter.from_nested_dict(completer_dict)

    # Setup Command Prompt
    while True:
        path = session.prompt("jnprsr> ", completer=completer)
        path = path.replace("show configuration ", "")
        try:
            subast = get_sub_tree(ast, path)
            print(render_config_from_ast(subast))
        except ChildResolverError as e:
            print("Error", e)






from sys import stdin
import argparse


def _read_from_stdin(silent=False):
    if not silent:
        print("[Type CTRL+D or '!END' at a new line to end input]")
    input_data = ""
    for line in stdin:
        if line.startswith("!END"):
            break
        input_data += line
    return input_data

def _read_from_file(filename):
    with open(filename, 'r') as file:
        file_content = file.read()

    return file_content



from anytree import Resolver, RootResolverError, ChildResolverError


class JunosResolver(Resolver):
    def get(self, node, path):
        node, parts = self._Resolver__start(node, path, self._Resolver__cmp)
        #print(parts)
        not_resolved_indicator = False
        for i in range(0, len(parts)):
            part = parts[i]

            # Attempt to resolve by assuming empty space is not a separator, but part of the node name
            if not_resolved_indicator:
                not_resolved_indicator = False
                # Prepend previous failed attempt at resolving to current item
                part = parts[i-1] + " " + part

            if part == "..":
                parent = node.parent
                if parent is None:
                    raise RootResolverError(node)
                node = parent
            elif part in ("", "."):
                pass
            else:
                try:
                    node = self._Resolver__get(node, part)
                except ChildResolverError as e:
                    # Indicate reference could not be resolved.
                    not_resolved_indicator = True

                    if i == len(parts)-1:
                        raise e

        return node


import unittest
import jnprsr.utils

class TestASTGeneration(unittest.TestCase):
    def read_test_file(self, name):
        with open(name, "r") as f:
            return f.read()

    def test_parser(self):
        ast = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        self.assertEqual(ast.name, "root")
        self.assertEqual(len(ast.children), 3)
        self.assertEqual(list(map(lambda x: x.name, ast.children)), ["system", "protocols", "interfaces"])
        self.assertEqual(list(map(lambda x: x.name, ast.children[2].children)), ["ge-0/0/0"])

    def test_config_renderer(self):
        ast = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        rendered_config = jnprsr.utils.render_config_from_ast(ast)
        self.assertEqual(self.read_test_file("test-conf1.txt"), rendered_config)

    def test_ascii_tree_renderer(self):
        ast = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        expected_ascii_tree = self.read_test_file("ascii-tree1.txt")
        self.assertEqual(expected_ascii_tree, jnprsr.utils.render_ascii_tree_from_ast(ast))


import argparse

import jnprsr
from jnprsr.cli.cliutils import _read_from_file


def _argparser_simple_two_files():
    parser = argparse.ArgumentParser(
        prog="jnprsr-merge",
        description='Merges a given Juniper configuration file \'file2\' onto \'file1\'',
        epilog="jnprsr is a Parser for Juniper Configuration Files"
    )
    parser.add_argument('file1', help="Configuration file1, file that will be the merge target")
    parser.add_argument('file2', help="Configuration file2, file that will be the merge source")
    args = parser.parse_args()
    return args

def merge():
    args = _argparser_simple_two_files()
    file1 = _read_from_file(args.file1)
    file2 = _read_from_file(args.file2)

    # Generate AST
    ast1 = jnprsr.get_ast(file1)
    ast2 = jnprsr.get_ast(file2)

    # Merge
    ast3 = jnprsr.merge(ast1, ast2)

    # Render Config again
    merged_config = jnprsr.render_config_from_ast(ast3)

    print(merged_config)


import unittest
import jnprsr.utils

class TestASTMerge(unittest.TestCase):
    def read_test_file(self, name):
        with open(name, "r") as f:
            return f.read()

    def test_1(self):
        ast1 = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        ast2 = jnprsr.utils.get_ast(self.read_test_file("test-conf2.txt"))
        ast3 = jnprsr.utils.merge(ast1, ast2)
        out = jnprsr.utils.render_config_from_ast(ast3)
        self.assertEqual(self.read_test_file("test-conf1-conf2-merged.txt"), out)

    def test_2(self):
        ast1 = jnprsr.utils.get_ast(self.read_test_file("test-conf1.txt"))
        ast2 = jnprsr.utils.get_ast(self.read_test_file("test-conf3.txt"))
        ast3 = jnprsr.utils.merge(ast1, ast2)
        out = jnprsr.utils.render_ascii_tree_from_ast(ast3)
        print(out)
        print(jnprsr.utils.render_config_from_ast(ast3))

    def test_3(self):
        ast1 = jnprsr.utils.get_ast(self.read_test_file("test-conf4.txt"))
        ast2 = jnprsr.utils.get_ast(self.read_test_file("test-conf5.txt"))
        ast3 = jnprsr.utils.merge(ast1, ast2)
        out = jnprsr.utils.render_config_from_ast(ast3)
        self.assertEqual(self.read_test_file("test-conf4-conf5-merged.txt"), out)